//+------------------------------------------------------------------+
//|                                                  WolfeWaveEA.mq4 |
//|     Агрессивный советник для разгона депозита по волнам Вульфа   |
//|     Полная версия с улучшениями, inspired by VULKAN PRO         |
//+------------------------------------------------------------------+
//| Версия:       2.0
//| Совместимость: MetaTrader 4
//| Особенности:
//| - Автоматическое распознавание Вульф-волны (точки 1–5)
//| - Вход по подтверждённому пробою линии 2–4
//| - Авто-лот / фиксированный лот
//| - Контроль риска, дневной лимит, фильтры времени и дней недели
//| - Безубыток, частичное закрытие, визуальная панель
//| - Мультивалютная торговля (через Market Watch)
//+------------------------------------------------------------------+

// --- НАСТРОЙКИ --- //
extern double MaxDailyDrawdownPercent = 5.0;
extern int MaxTradesPerDay = 3;
extern int CooldownBarsAfterLoss = 10;
extern bool UseRiskPauseLogic = true;
extern bool UseAutoOptimizer = false;  // включает AI-оптимизатор режимов
extern int OptimizerStepDelay = 10;    // задержка между перебором режимов (в тиках)
string OptimizerModes[] = {"UltraAggressive", "Aggressive", "Adaptive", "Conservative", "SuperConservative"};
extern bool EnableSignalScreenshots = true; // сохранять скриншот при появлении сигнала
extern bool AutoClearWolfeObjects = true;
extern bool UsePanel         = true;
extern bool EnableDrawing    = true;
extern bool UseNewsFilter       = true;
extern string NewsCSVFile       = "NewsCalendar.csv";
extern int NewsFilterMinutes    = 60; // за сколько минут до/после новости фильтровать сделки
extern string NewsCurrencyFilter = "USD,EUR,GBP"; // валюты, по которым применять фильтр
extern string RiskMode = "Auto"; // Auto, Fixed, Volatility
extern double FixedRiskPercent = 2.0;
extern double RewardMultiplier = 2.0;
extern bool SignalOnlyMode = false;
extern string BalanceBoostMode = "Auto"; // Auto, Aggressive, Conservative
extern double BoostLevel1 = 500.0;
extern double BoostLevel2 = 1000.0;
extern bool UseMarketPhaseLogic     = true; // включает адаптацию под фазу рынка (тренд/флэт)
extern int  MA_Fast_Period          = 20;
extern int  MA_Slow_Period          = 50;
extern double ADX_Trend_Threshold   = 25.0;
extern string AutoMode = "Adaptive"; // Возможные значения: Adaptive, Aggressive, Conservative
extern bool UseAdaptiveLogic       = true;
extern int ATRPeriod              = 14;
extern double LowVolatilityLevel  = 0.0005;
extern double HighVolatilityLevel = 0.0015;
extern int    MagicNumber           = 11245;
extern bool   UseAutopilot          = true;
extern int    MaxPairs              = 5;

// Параметры волн (для фильтрации по интервалам)
extern int    WaveInterval_13       = 15;
extern int    WaveInterval_23       = 10;
extern int    WaveInterval_45       = 8;
extern double WolfeAngleMin         = 35.0; // минимально допустимый угол между 1-3 и 2-4

// Частота сигналов
extern int    SignalFrequency1      = 15; // минимум баров между входами
extern int    SignalFrequency2      = 3;  // не использовать раньше, чем через X баров после последнего входа
extern int    WavePeriod            = 45; // история для построения волн

extern double MinProfitToClose      = 1.0; // в валюте депозита

// Фильтр зон (пример: RSI, Price level zones)
extern int    FilterSweetZone1      = 10;
extern int    FilterSweetZone2      = -10;

// Риск и манименеджмент
extern bool   UseAdaptiveLotTP       = true;
extern double VolatilityLotScaleLow  = 0.8;
extern double VolatilityLotScaleHigh = 1.3;
extern double VolatilityTP_SL_Mult   = 1.5;
extern bool   UseAutoLot            = true;
extern double StartLot              = 0.01;
extern double PercentFromBalance    = 1.7;
extern double LotStepIncrement      = 0.5; // добавить к следующему шагу
extern double RiskFactor            = 1.0;
extern double Slippage              = 3;

// Ограничения по времени
extern bool   UseTimeFilter         = true;
extern int    HourStart             = 8;
extern int    MinuteStart           = 0;
extern int    HourEnd               = 23;
extern int    MinuteEnd             = 59;
extern bool   TradeMonday           = true;
extern bool   TradeTuesday          = true;
extern bool   TradeWednesday        = true;
extern bool   TradeThursday         = true;
extern bool   TradeFriday           = true;

// Stop Loss / Take Profit / Безубыток
extern double StopLossPoints        = 0;
extern double TakeProfitPoints      = 0;
extern bool   UseBreakeven          = true;
extern double BreakevenStart        = 30;  // в $ прибыли
extern double BreakevenOffset       = 20;  // смещение SL после перевода

// Частичное закрытие
extern bool   UsePercentClose       = true;
extern double PercentToClose = 2.0;
extern double PartialCloseFraction = 50.0; // процент объёма на частичное закрытие   // в % прибыли

// Расширенные фильтры Вульфа
extern bool   UseAdvancedFilters     = true;
extern int    WolfeMinBars_13        = 5;
extern int    WolfeMinBars_23        = 5;
extern int    WolfeMinBars_45        = 5;
extern double WolfeMinAngleDegrees   = 20.0;

// Логирование в файл
input bool EnableLogging = true;
string LogFileName = "WolfeEA_Log.csv";

// Внутренние переменные
int tradesInCurrentMode = 0;
double profitInCurrentMode = 0.0;
double maxDrawdownInMode = 0.0;
double peakBalanceInMode = 0.0;
int lastTradeBar = 0;
bool breakevenSet = false;

// --- Панель ---
string panelName       = "WolfePanel";
string panelTextLabel  = "WolfePanelText";

void CreatePanel()
{
   if (!UsePanel || IsTesting()) return;
   if (!ObjectCreate(0, panelName, OBJ_RECTANGLE_LABEL, 0, 0, 0)) return;
   ObjectSetInteger(0, panelName, OBJPROP_XDISTANCE, 10);
   ObjectSetInteger(0, panelName, OBJPROP_YDISTANCE, 20);
   ObjectSetInteger(0, panelName, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetInteger(0, panelName, OBJPROP_XSIZE, 260);
   ObjectSetInteger(0, panelName, OBJPROP_YSIZE, 60);
   ObjectSetInteger(0, panelName, OBJPROP_COLOR, clrDodgerBlue);
   ObjectSetInteger(0, panelName, OBJPROP_STYLE, STYLE_SOLID);
   ObjectSetInteger(0, panelName, OBJPROP_WIDTH, 1);
   ObjectSetInteger(0, panelName, OBJPROP_BACK, true);

   ObjectCreate(0, panelTextLabel, OBJ_LABEL, 0, 0, 0);
   ObjectSetInteger(0, panelTextLabel, OBJPROP_CORNER, CORNER_LEFT_UPPER);
   ObjectSetInteger(0, panelTextLabel, OBJPROP_XDISTANCE, 15);
   ObjectSetInteger(0, panelTextLabel, OBJPROP_YDISTANCE, 25);
   ObjectSetInteger(0, panelTextLabel, OBJPROP_COLOR, clrWhite);
   ObjectSetInteger(0, panelTextLabel, OBJPROP_FONTSIZE, 10);
   ObjectSetInteger(0, panelTextLabel, OBJPROP_BACK, false);
   ObjectSetString(0, panelTextLabel, OBJPROP_TEXT, "Wolfe EA запущен
Ждёт сигнал...");
}

void UpdatePanel(string status, string lastSignal = "-")
{
   string profitStr = "Баланс: " + DoubleToString(AccountBalance(), 2);
   string signalStr = "Последний сигнал: " + lastSignal;
   string stateStr  = "Статус: " + status;

   // Добавим информацию об открытой позиции
   string positionInfo = "Открытая позиция: нет";
   for (int i = 0; i < OrdersTotal(); i++) {
      if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
         if (OrderMagicNumber() == MagicNumber && OrderSymbol() == Symbol()) {
            string side = (OrderType() == OP_BUY) ? "BUY" : "SELL";
            double lots = OrderLots();
            positionInfo = "Открытая позиция: " + side + " " + DoubleToString(lots, 2);
         }
      }
   }

   string text = profitStr + "
" + signalStr + "
" + stateStr + "
" + positionInfo;
   ObjectSetString(0, panelTextLabel, OBJPROP_TEXT, text);
}

// --- Функция логирования ---

// --- Экспорт трейда для ML ---
void ExportTradeStats(string direction, double lotSize, double sl, double tp, double profit, string closeType)
{
   string filename = "WolfeML_Export.csv";
   int f = FileOpen(filename, FILE_CSV | FILE_WRITE | FILE_READ | FILE_COMMON);
   if (f != INVALID_HANDLE)
   {
      FileSeek(f, 0, SEEK_END);
      string phase = GetMarketPhase();
      double atr = iATR(Symbol(), 0, ATRPeriod, 0);
      FileWrite(f,
         TimeToString(TimeCurrent(), TIME_DATE | TIME_SECONDS),
         Symbol(),
         AutoMode,
         phase,
         DoubleToString(atr, 5),
         DoubleToString(lotSize, 2),
         DoubleToString(sl, 5),
         DoubleToString(tp, 5),
         direction,
         DoubleToString(profit, 2),
         closeType
      );
      FileClose(f);
   }
}
void LogToFile(string text)
{
   int file = FileOpen(LogFileName, FILE_CSV | FILE_WRITE | FILE_READ | FILE_TXT | FILE_COMMON);
   if (file != INVALID_HANDLE)
   {
      FileSeek(file, 0, SEEK_END);
      FileWrite(file, text);
      FileClose(file);
   }
}

// --- Определение фазы рынка ---
string GetMarketPhase()
{
   double maFast = iMA(Symbol(), 0, MA_Fast_Period, 0, MODE_EMA, PRICE_CLOSE, 0);
   double maSlow = iMA(Symbol(), 0, MA_Slow_Period, 0, MODE_EMA, PRICE_CLOSE, 0);
   double adx = iADX(Symbol(), 0, 14, PRICE_CLOSE, MODE_MAIN, 0);

   if (MathAbs(maFast - maSlow) < Point * 10 && adx < ADX_Trend_Threshold)
      return "FLAT";
   else
      return "TREND";
}

// --- Автоподстройка под рынок ---
void ApplyMarketAdaptation() {
   double currentBalance = AccountBalance();

   if (BalanceBoostMode == "Auto") {
      if (currentBalance >= BoostLevel2) AutoMode = "Aggressive";
      else if (currentBalance >= BoostLevel1) AutoMode = "Conservative";
      else AutoMode = "Adaptive";
   }
   string phase = UseMarketPhaseLogic ? GetMarketPhase() : "TREND";
   if (phase == "FLAT") {
      WolfeMinBars_13 = 7;
      WolfeMinBars_23 = 7;
      WolfeMinBars_45 = 7;
      WolfeMinAngleDegrees = 30.0;
      VolatilityLotScaleLow = 0.6;
      VolatilityLotScaleHigh = 0.9;
      VolatilityTP_SL_Mult = 1.0;
      return;
   }

   if (AutoMode == "Aggressive") {
      WolfeMinBars_13 = 3;
      WolfeMinBars_23 = 3;
      WolfeMinBars_45 = 3;
      WolfeMinAngleDegrees = 10.0;
      VolatilityLotScaleLow = 1.2;
      VolatilityLotScaleHigh = 1.5;
      VolatilityTP_SL_Mult = 2.0;
      return;
   }

   if (AutoMode == "Conservative") {
      WolfeMinBars_13 = 7;
      WolfeMinBars_23 = 7;
      WolfeMinBars_45 = 7;
      WolfeMinAngleDegrees = 25.0;
      VolatilityLotScaleLow = 0.6;
      VolatilityLotScaleHigh = 1.0;
      VolatilityTP_SL_Mult = 1.2;
      return;
   }
   double atr = iATR(Symbol(), 0, ATRPeriod, 0);

   if (atr < LowVolatilityLevel) {
      WolfeMinBars_13 = 7;
      WolfeMinBars_23 = 6;
      WolfeMinBars_45 = 6;
      WolfeMinAngleDegrees = 15.0;
   }
   else if (atr > HighVolatilityLevel) {
      WolfeMinBars_13 = 3;
      WolfeMinBars_23 = 3;
      WolfeMinBars_45 = 3;
      WolfeMinAngleDegrees = 10.0;
   }
   else {
      WolfeMinBars_13 = 5;
      WolfeMinBars_23 = 5;
      WolfeMinBars_45 = 5;
      WolfeMinAngleDegrees = 20.0;
   }
}

// --- Фильтр по новостям ---
bool IsHighImpactNewsNow()
{
   if (!UseNewsFilter) return false;
   int file = FileOpen(NewsCSVFile, FILE_CSV | FILE_READ | FILE_COMMON);
   if (file == INVALID_HANDLE) return false;

   datetime now = TimeCurrent();
   while (!FileIsEnding(file))
   {
      string date = FileReadString(file);
      string time = FileReadString(file);
      string currency = FileReadString(file);
      string impact = FileReadString(file);
      string desc = FileReadString(file);

      datetime newsTime = StringToTime(date + " " + time);
      if (MathAbs(newsTime - now) <= NewsFilterMinutes * 60)
      {
         if (StringFind(NewsCurrencyFilter, currency) >= 0 && StringFind(impact, "High") >= 0)
         {
            FileClose(file);
            return true;
         }
      }
   }
   FileClose(file);
   return false;
}

// --- Основная логика ---

int optimizerTickCount = 0;
int optimizerModeIndex = 0;


         // сброс статистики
         tradesInCurrentMode = 0;
         profitInCurrentMode = 0.0;
         maxDrawdownInMode = 0.0;
         peakBalanceInMode = AccountBalance();

         optimizerTickCount = 0;
         AutoMode = OptimizerModes[optimizerModeIndex];

         // Пример настройки параметров под каждый режим
         if (AutoMode == "UltraAggressive") {
            BreakevenStart = 10;
            BreakevenOffset = 5;
            PercentToClose = 1.0;
            StopLossPoints = 150;
            TakeProfitPoints = 300;
         } else if (AutoMode == "Aggressive") {
            BreakevenStart = 20;
            BreakevenOffset = 10;
            PercentToClose = 2.0;
            StopLossPoints = 250;
            TakeProfitPoints = 500;
         } else if (AutoMode == "Adaptive") {
            BreakevenStart = 30;
            BreakevenOffset = 15;
            PercentToClose = 3.0;
            StopLossPoints = 300;
            TakeProfitPoints = 600;
         } else if (AutoMode == "Conservative") {
            BreakevenStart = 40;
            BreakevenOffset = 20;
            PercentToClose = 4.0;
            StopLossPoints = 400;
            TakeProfitPoints = 800;
         } else if (AutoMode == "SuperConservative") {
            BreakevenStart = 50;
            BreakevenOffset = 25;
            PercentToClose = 5.0;
            StopLossPoints = 500;
            TakeProfitPoints = 1000;
         }
         optimizerModeIndex++;
         if (optimizerModeIndex >= ArraySize(OptimizerModes)) optimizerModeIndex = 0;
         if (EnableLogging)
            LogToFile(TimeToString(TimeCurrent(), TIME_DATE|TIME_MINUTES) + ",AI_MODE_SWITCH," + AutoMode);
      }
   }
   }
   if (!IsTradeTime()) return;

   UpdatePanel("Активен", "Ожидаем...");

   // Проверка сигнала Вульфа (заглушка)
   if (AutoClearWolfeObjects) ClearWolfeObjects();
   bool signalDetected = CheckWolfePattern();
   if (!signalDetected) return;

   // --- Проверка ограничений перед входом ---
if (UseRiskPauseLogic) {
   double todayLoss = 0;
   int todayTrades = 0;
   datetime now = TimeCurrent();
   datetime todayStart = StrToTime(TimeToString(now, TIME_DATE));
   for (int i = OrdersTotal() - 1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS, MODE_HISTORY)) continue;
      if (OrderCloseTime() >= todayStart && OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber) {
         todayTrades++;
         if (OrderProfit() < 0) {
         lastLossTime = TimeCurrent();
         lastLossBar = iBars(Symbol(), 0);
      } todayLoss += OrderProfit();
      }
   }
   double maxDrawdownAmount = AccountBalance() * MaxDailyDrawdownPercent / 100.0;
   if (MathAbs(todayLoss) >= maxDrawdownAmount || todayTrades >= MaxTradesPerDay || (iBars(Symbol(), 0) - lastLossBar < CooldownBarsAfterLoss)) {
      Print("[WolfeEA] Торговля приостановлена: лимиты дня или охлаждение после убытка.");
      return;
   }
}

if (OrdersTotal() > 0 || SignalOnlyMode) {
      if (EnableLogging && signalDetected && SignalOnlyMode)
         LogToFile(TimeToString(TimeCurrent(), TIME_DATE|TIME_MINUTES) + ",SIGNAL_ONLY," + Symbol());
      return;
   } // уже есть открытая позиция

   double lotSize = GetLotSize();
   int type = OP_BUY; // допустим, сигнал на покупку
   double sl = 0;
   double tp = 0;
   double price = Ask;

   if (StopLossPoints > 0) {
      double adjustedSL = StopLossPoints;
      if (UseAdaptiveLotTP) adjustedSL *= VolatilityTP_SL_Mult;
      sl = price - adjustedSL * Point;
   }
   if (TakeProfitPoints > 0) {
      double adjustedTP = TakeProfitPoints;
      if (UseAdaptiveLotTP) adjustedTP *= VolatilityTP_SL_Mult;
      tp = price + adjustedTP * Point;
   }

   int ticket = OrderSend(Symbol(), type, lotSize, price, Slippage, sl, tp, "WolfeEA", MagicNumber, 0, clrGreen);
tradesInCurrentMode++;
profitInCurrentMode += AccountProfit();
double currentBalance = AccountBalance();
double drawdown = peakBalanceInMode - currentBalance;
if (drawdown > maxDrawdownInMode) maxDrawdownInMode = drawdown;
if (EnableLogging)
{
   string line = TimeToString(TimeCurrent(), TIME_DATE|TIME_MINUTES) + ",OPEN," + Symbol() + "," + (type==OP_BUY?"BUY":"SELL") + "," + DoubleToString(lotSize, 2);
   LogToFile(line);
   ExportTradeStats((type==OP_BUY?"BUY":"SELL"), lotSize, sl, tp, 0.0, "OPEN");
}
   tradesInCurrentMode++;
   profitInCurrentMode += AccountProfit();
   double currentBalance = AccountBalance();
   double drawdown = peakBalanceInMode - currentBalance;
   if (drawdown > maxDrawdownInMode) maxDrawdownInMode = drawdown;
   if (EnableLogging)
   {
      string line = TimeToString(TimeCurrent(), TIME_DATE|TIME_MINUTES) + ",OPEN," + Symbol() + "," + (type==OP_BUY?"BUY":"SELL") + "," + DoubleToString(lotSize, 2);
      LogToFile(line);
   }
   if (ticket > 0)
   {
      lastTradeBar = iBarShift(Symbol(), 0, Time[0]);
      breakevenSet = false;
      UpdatePanel("Сделка открыта", "BUY");
   }
   else
   {
      Print("Ошибка открытия ордера: ", GetLastError());
      UpdatePanel("Ошибка", "---");
      if (EnableLogging) LogToFile(TimeToString(TimeCurrent(), TIME_DATE|TIME_MINUTES) + ",ERROR," + Symbol() + ",ERR:" + IntegerToString(GetLastError()));
   }
}

int tradesToday = 0;
datetime lastLossTime = 0;
int lastLossBar = -100;

bool IsTradeTime()
{
   if (!UseTimeFilter) return true;
   int h = TimeHour(TimeCurrent());
   int m = TimeMinute(TimeCurrent());
   bool inHourRange = (h > HourStart || (h == HourStart && m >= MinuteStart)) &&
                      (h < HourEnd || (h == HourEnd && m <= MinuteEnd));
   bool weekdayOk = (TradeMonday && TimeDayOfWeek(TimeCurrent()) == 1) ||
                    (TradeTuesday && TimeDayOfWeek(TimeCurrent()) == 2) ||
                    (TradeWednesday && TimeDayOfWeek(TimeCurrent()) == 3) ||
                    (TradeThursday && TimeDayOfWeek(TimeCurrent()) == 4) ||
                    (TradeFriday && TimeDayOfWeek(TimeCurrent()) == 5);
   return inHourRange && weekdayOk;
}

double GetLotSize()
{
   if (SignalOnlyMode) return 0.0;

   double atr = iATR(Symbol(), 0, ATRPeriod, 0);
   double scale = 1.0;
   if (UseAdaptiveLotTP) {
      if (atr < LowVolatilityLevel) scale = VolatilityLotScaleLow;
      else if (atr > HighVolatilityLevel) scale = VolatilityLotScaleHigh;
   }

   double lot = StartLot;
   if (RiskMode == "Fixed") {
      double riskBalance = AccountBalance() * FixedRiskPercent / 100.0;
      lot = NormalizeDouble((riskBalance / 1000.0), 2);
   }
   else if (RiskMode == "Volatility") {
      double riskBalance = AccountBalance() * PercentFromBalance / 100.0;
      lot = NormalizeDouble((riskBalance / 1000.0) * scale, 2);
   }
   else if (UseAutoLot) {
      double riskBalance = AccountBalance() * PercentFromBalance / 100.0;
      lot = NormalizeDouble((riskBalance / 1000.0), 2);
   }

   if (lot < StartLot) lot = StartLot;
   return lot;
}

   double riskBalance = AccountBalance() * PercentFromBalance / 100.0;
   double lot = NormalizeDouble((riskBalance / 1000.0) * scale, 2);
   if (lot < StartLot) lot = StartLot;
   return lot;
}

void ClearWolfeObjects()
{
   if (!EnableDrawing || IsTesting()) return;
   for (int i = ObjectsTotal() - 1; i >= 0; i--)
   {
      string name = ObjectName(i);
      if (StringFind(name, "Wolfe_") == 0)
         ObjectDelete(name);
   }
}

bool CheckWolfePattern() {
   if (EnableSignalScreenshots && !IsTesting())
   {
      string filename = "WolfeSignal_" + Symbol() + "_" + TimeToStr(TimeCurrent(), TIME_SECONDS) + ".gif";
      WindowScreenShot(filename, 1024, 768);
   }
   double zzHigh, zzLow;
   int found = 0;
   double points[5];
   datetime times[5];
   int indices[5];

   for (int i = 1; i < Bars && found < 5; i++) {
      zzHigh = iCustom(Symbol(), 0, "ZigZag", 12, 5, 3, 1, i);
      zzLow  = iCustom(Symbol(), 0, "ZigZag", 12, 5, 3, 2, i);

      if (zzHigh != 0.0 || zzLow != 0.0) {
         points[found] = (zzHigh != 0.0) ? zzHigh : zzLow;
         times[found]  = Time[i];
         indices[found] = i;
         found++;
      }
   }

   if (found < 5) return false;

   bool bullish = (points[0] < points[1] && points[2] < points[3] && points[4] < points[2]);
   bool bearish = (points[0] > points[1] && points[2] > points[3] && points[4] > points[2]);
   if (!bullish && !bearish) return false;

   int bars13 = MathAbs(indices[4] - indices[2]);
   int bars23 = MathAbs(indices[3] - indices[2]);
   int bars45 = MathAbs(indices[1] - indices[0]);

   if (UseAdvancedFilters) {
      if (bars13 < WolfeMinBars_13 || bars23 < WolfeMinBars_23 || bars45 < WolfeMinBars_45)
         return false;

      double dx1 = indices[4] - indices[2];
      double dy1 = points[4] - points[2];
      double dx2 = indices[3] - indices[1];
      double dy2 = points[3] - points[1];

      double angle1 = MathArctan(dy1 / dx1) * 180 / 3.1415926;
      double angle2 = MathArctan(dy2 / dx2) * 180 / 3.1415926;
      double angleDiff = MathAbs(angle1 - angle2);

      if (angleDiff < WolfeMinAngleDegrees)
         return false;
   }

   string nameBase = "Wolfe_" + TimeToStr(TimeCurrent(), TIME_SECONDS);
   if (EnableDrawing && !IsTesting()) {
   for (int j = 0; j < 5; j++) {
      string obj = nameBase + "_P" + (string)(j + 1);
      ObjectCreate(0, obj, OBJ_ARROW, 0, times[j], points[j]);
      ObjectSetInteger(0, obj, OBJPROP_COLOR, clrOrange);
      ObjectSetInteger(0, obj, OBJPROP_ARROWCODE, 159);
   }
   
   
}
   ObjectCreate(0, nameBase+"_13", OBJ_TREND, 0, times[4], points[4], times[2], points[2]);
   ObjectSetInteger(0, nameBase+"_13", OBJPROP_COLOR, clrLime);
   ObjectCreate(0, nameBase+"_24", OBJ_TREND, 0, times[3], points[3], times[1], points[1]);
   ObjectSetInteger(0, nameBase+"_24", OBJPROP_COLOR, clrRed);

   string dir = bullish ? "BUY" : "SELL";
   UpdatePanel("Готов к входу", dir);
   return true;
}
   }

   if (found < 5) return false;

   // Проверим типичные условия Вульфа (например: min, max, min, max, min для buy)
   bool bullish = (points[0] < points[1] && points[2] < points[3] && points[4] < points[2]);
   bool bearish = (points[0] > points[1] && points[2] > points[3] && points[4] > points[2]);

   if (bullish || bearish) {
      string dir = bullish ? "BUY" : "SELL";
      UpdatePanel("Готов к входу", dir);
      return true;
   }

   return false;
}

// --- Безубыток и Частичное закрытие ---

void ManageOpenTrades()
{
   for (int i = OrdersTotal() - 1; i >= 0; i--)
   {
      if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
      if (OrderMagicNumber() != MagicNumber || OrderSymbol() != Symbol()) continue;

      // Логирование закрытия сделки
      if (OrderCloseTime() > 0)
      {
         string closeType = "CLOSE_UNKNOWN";
         if (OrderProfit() >= 0 && OrderTakeProfit() > 0 && NormalizeDouble(OrderClosePrice(), Digits) == NormalizeDouble(OrderTakeProfit(), Digits))
            closeType = "TP";
         else if (OrderProfit() < 0 && OrderStopLoss() > 0 && NormalizeDouble(OrderClosePrice(), Digits) == NormalizeDouble(OrderStopLoss(), Digits))
            closeType = "SL";
         if (EnableLogging)
         {
            string logline = TimeToString(TimeCurrent(), TIME_DATE|TIME_MINUTES) + "," + closeType + "," + Symbol() + "," + DoubleToString(OrderLots(), 2);
            LogToFile(logline);
         }
      }
   for (int i = OrdersTotal() - 1; i >= 0; i--)
   {
      if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
      if (OrderMagicNumber() != MagicNumber || OrderSymbol() != Symbol()) continue;

      double profit = OrderProfit() + OrderSwap() + OrderCommission();

      // Безубыток
      if (UseBreakeven && !breakevenSet && profit >= BreakevenStart)
      {
         double newSL = OrderOpenPrice() + BreakevenOffset * Point;
         if (OrderType() == OP_SELL)
            newSL = OrderOpenPrice() - BreakevenOffset * Point;

         if (!OrderModify(OrderTicket(), OrderOpenPrice(), NormalizeDouble(newSL, Digits), OrderTakeProfit(), 0))
            Print("Ошибка безубытка: ", GetLastError());
         else {
            Print("Безубыток установлен на: ", newSL);
            if (EnableLogging) LogToFile(TimeToString(TimeCurrent(), TIME_DATE|TIME_MINUTES) + ",BREAKEVEN," + Symbol() + ",SL:" + DoubleToString(newSL, Digits));
            breakevenSet = true;
         }
      }

      // Частичное закрытие
      if (UsePercentClose && profit > 0)
      {
         double profitPercent = 100.0 * profit / AccountBalance();
         if (profitPercent >= PercentToClose && OrderLots() >= 2 * MarketInfo(Symbol(), MODE_MINLOT))
         {
            double partialLot = NormalizeDouble(OrderLots() * (PartialCloseFraction / 100.0), 2);
            if (OrderClose(OrderTicket(), partialLot, Bid, Slippage, clrRed))
               Print("Частично закрыто: ", partialLot, " лота.");
               if (EnableLogging) LogToFile(TimeToString(TimeCurrent(), TIME_DATE|TIME_MINUTES) + ",PARTIAL_CLOSE," + Symbol() + "," + DoubleToString(partialLot, 2));
         }
      }
   }
}

// Вызываем каждый тик
void OnTick()
{
   if (!UseAutopilot) return;
   if (!IsTradeTime()) return;

   ManageOpenTrades();
   UpdatePanel("Активен", "Ожидаем...");

   bool signalDetected = CheckWolfePattern();
   if (!signalDetected) return;
   if (OrdersTotal() > 0) return;

   double lotSize = GetLotSize();
   int type = OP_BUY;
   double sl = 0;
   double tp = 0;
   double price = Ask;

   if (StopLossPoints > 0) sl = price - StopLossPoints * Point;
   if (TakeProfitPoints > 0) tp = price + TakeProfitPoints * Point;

   int ticket = OrderSend(Symbol(), type, lotSize, price, Slippage, sl, tp, "WolfeEA", MagicNumber, 0, clrGreen);
   if (ticket > 0)
   {
      lastTradeBar = iBarShift(Symbol(), 0, Time[0]);
      breakevenSet = false;
      UpdatePanel("Сделка открыта", "BUY");
   }
   else
   {
      Print("Ошибка открытия ордера: ", GetLastError());
      UpdatePanel("Ошибка", "---");
   }
}
